/* -*- buffer-read-only: t -*-
   !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
   This file is built by opcode.pl from its data.
   Any changes made here will be lost!
 */

PERL_CALLCONV OP *Perl_do_kv(pTHX);
PERL_CALLCONV OP *Perl_pp_aassign(pTHX);
PERL_CALLCONV OP *Perl_pp_abs(pTHX);
PERL_CALLCONV OP *Perl_pp_accept(pTHX);
PERL_CALLCONV OP *Perl_pp_add(pTHX);
PERL_CALLCONV OP *Perl_pp_aeach(pTHX);
PERL_CALLCONV OP *Perl_pp_aelem(pTHX);
PERL_CALLCONV OP *Perl_pp_aelemfast(pTHX);
PERL_CALLCONV OP *Perl_pp_akeys(pTHX);
PERL_CALLCONV OP *Perl_pp_alarm(pTHX);
PERL_CALLCONV OP *Perl_pp_and(pTHX);
PERL_CALLCONV OP *Perl_pp_anoncode(pTHX);
PERL_CALLCONV OP *Perl_pp_anonconst(pTHX);
PERL_CALLCONV OP *Perl_pp_anonhash(pTHX);
PERL_CALLCONV OP *Perl_pp_anonlist(pTHX);
PERL_CALLCONV OP *Perl_pp_argcheck(pTHX);
PERL_CALLCONV OP *Perl_pp_argdefelem(pTHX);
PERL_CALLCONV OP *Perl_pp_argelem(pTHX);
PERL_CALLCONV OP *Perl_pp_aslice(pTHX);
PERL_CALLCONV OP *Perl_pp_atan2(pTHX);
PERL_CALLCONV OP *Perl_pp_av2arylen(pTHX);
PERL_CALLCONV OP *Perl_pp_avhvswitch(pTHX);
PERL_CALLCONV OP *Perl_pp_backtick(pTHX);
PERL_CALLCONV OP *Perl_pp_bind(pTHX);
PERL_CALLCONV OP *Perl_pp_binmode(pTHX);
PERL_CALLCONV OP *Perl_pp_bit_and(pTHX);
PERL_CALLCONV OP *Perl_pp_bit_or(pTHX);
PERL_CALLCONV OP *Perl_pp_bless(pTHX);
PERL_CALLCONV OP *Perl_pp_break(pTHX);
PERL_CALLCONV OP *Perl_pp_caller(pTHX);
PERL_CALLCONV OP *Perl_pp_chdir(pTHX);
PERL_CALLCONV OP *Perl_pp_chop(pTHX);
PERL_CALLCONV OP *Perl_pp_chown(pTHX);
PERL_CALLCONV OP *Perl_pp_chr(pTHX);
PERL_CALLCONV OP *Perl_pp_chroot(pTHX);
PERL_CALLCONV OP *Perl_pp_clonecv(pTHX);
PERL_CALLCONV OP *Perl_pp_close(pTHX);
PERL_CALLCONV OP *Perl_pp_closedir(pTHX);
PERL_CALLCONV OP *Perl_pp_complement(pTHX);
PERL_CALLCONV OP *Perl_pp_concat(pTHX);
PERL_CALLCONV OP *Perl_pp_cond_expr(pTHX);
PERL_CALLCONV OP *Perl_pp_const(pTHX);
PERL_CALLCONV OP *Perl_pp_continue(pTHX);
PERL_CALLCONV OP *Perl_pp_coreargs(pTHX);
PERL_CALLCONV OP *Perl_pp_crypt(pTHX);
PERL_CALLCONV OP *Perl_pp_dbmopen(pTHX);
PERL_CALLCONV OP *Perl_pp_dbstate(pTHX);
PERL_CALLCONV OP *Perl_pp_defined(pTHX);
PERL_CALLCONV OP *Perl_pp_delete(pTHX);
PERL_CALLCONV OP *Perl_pp_die(pTHX);
PERL_CALLCONV OP *Perl_pp_divide(pTHX);
PERL_CALLCONV OP *Perl_pp_each(pTHX);
PERL_CALLCONV OP *Perl_pp_ehostent(pTHX);
PERL_CALLCONV OP *Perl_pp_enter(pTHX);
PERL_CALLCONV OP *Perl_pp_entereval(pTHX);
PERL_CALLCONV OP *Perl_pp_entergiven(pTHX);
PERL_CALLCONV OP *Perl_pp_enteriter(pTHX);
PERL_CALLCONV OP *Perl_pp_enterloop(pTHX);
PERL_CALLCONV OP *Perl_pp_entersub(pTHX);
PERL_CALLCONV OP *Perl_pp_entertry(pTHX);
PERL_CALLCONV OP *Perl_pp_enterwhen(pTHX);
PERL_CALLCONV OP *Perl_pp_enterwrite(pTHX);
PERL_CALLCONV OP *Perl_pp_eof(pTHX);
PERL_CALLCONV OP *Perl_pp_eq(pTHX);
PERL_CALLCONV OP *Perl_pp_exec(pTHX);
PERL_CALLCONV OP *Perl_pp_exists(pTHX);
PERL_CALLCONV OP *Perl_pp_exit(pTHX);
PERL_CALLCONV OP *Perl_pp_fc(pTHX);
PERL_CALLCONV OP *Perl_pp_fileno(pTHX);
PERL_CALLCONV OP *Perl_pp_flip(pTHX);
PERL_CALLCONV OP *Perl_pp_flock(pTHX);
PERL_CALLCONV OP *Perl_pp_flop(pTHX);
PERL_CALLCONV OP *Perl_pp_fork(pTHX);
PERL_CALLCONV OP *Perl_pp_formline(pTHX);
PERL_CALLCONV OP *Perl_pp_ftis(pTHX);
PERL_CALLCONV OP *Perl_pp_ftlink(pTHX);
PERL_CALLCONV OP *Perl_pp_ftrowned(pTHX);
PERL_CALLCONV OP *Perl_pp_ftrread(pTHX);
PERL_CALLCONV OP *Perl_pp_fttext(pTHX);
PERL_CALLCONV OP *Perl_pp_fttty(pTHX);
PERL_CALLCONV OP *Perl_pp_ge(pTHX);
PERL_CALLCONV OP *Perl_pp_gelem(pTHX);
PERL_CALLCONV OP *Perl_pp_getc(pTHX);
PERL_CALLCONV OP *Perl_pp_getlogin(pTHX);
PERL_CALLCONV OP *Perl_pp_getpeername(pTHX);
PERL_CALLCONV OP *Perl_pp_getpgrp(pTHX);
PERL_CALLCONV OP *Perl_pp_getppid(pTHX);
PERL_CALLCONV OP *Perl_pp_getpriority(pTHX);
PERL_CALLCONV OP *Perl_pp_ggrent(pTHX);
PERL_CALLCONV OP *Perl_pp_ghostent(pTHX);
PERL_CALLCONV OP *Perl_pp_glob(pTHX);
PERL_CALLCONV OP *Perl_pp_gmtime(pTHX);
PERL_CALLCONV OP *Perl_pp_gnetent(pTHX);
PERL_CALLCONV OP *Perl_pp_goto(pTHX);
PERL_CALLCONV OP *Perl_pp_gprotoent(pTHX);
PERL_CALLCONV OP *Perl_pp_gpwent(pTHX);
PERL_CALLCONV OP *Perl_pp_grepstart(pTHX);
PERL_CALLCONV OP *Perl_pp_grepwhile(pTHX);
PERL_CALLCONV OP *Perl_pp_gservent(pTHX);
PERL_CALLCONV OP *Perl_pp_gt(pTHX);
PERL_CALLCONV OP *Perl_pp_gv(pTHX);
PERL_CALLCONV OP *Perl_pp_gvsv(pTHX);
PERL_CALLCONV OP *Perl_pp_helem(pTHX);
PERL_CALLCONV OP *Perl_pp_hintseval(pTHX);
PERL_CALLCONV OP *Perl_pp_hslice(pTHX);
PERL_CALLCONV OP *Perl_pp_i_add(pTHX);
PERL_CALLCONV OP *Perl_pp_i_divide(pTHX);
PERL_CALLCONV OP *Perl_pp_i_eq(pTHX);
PERL_CALLCONV OP *Perl_pp_i_ge(pTHX);
PERL_CALLCONV OP *Perl_pp_i_gt(pTHX);
PERL_CALLCONV OP *Perl_pp_i_le(pTHX);
PERL_CALLCONV OP *Perl_pp_i_lt(pTHX);
PERL_CALLCONV OP *Perl_pp_i_modulo(pTHX);
PERL_CALLCONV OP *Perl_pp_i_multiply(pTHX);
PERL_CALLCONV OP *Perl_pp_i_ncmp(pTHX);
PERL_CALLCONV OP *Perl_pp_i_ne(pTHX);
PERL_CALLCONV OP *Perl_pp_i_negate(pTHX);
PERL_CALLCONV OP *Perl_pp_i_subtract(pTHX);
PERL_CALLCONV OP *Perl_pp_index(pTHX);
PERL_CALLCONV OP *Perl_pp_int(pTHX);
PERL_CALLCONV OP *Perl_pp_introcv(pTHX);
PERL_CALLCONV OP *Perl_pp_ioctl(pTHX);
PERL_CALLCONV OP *Perl_pp_iter(pTHX);
PERL_CALLCONV OP *Perl_pp_join(pTHX);
PERL_CALLCONV OP *Perl_pp_kvaslice(pTHX);
PERL_CALLCONV OP *Perl_pp_kvhslice(pTHX);
PERL_CALLCONV OP *Perl_pp_last(pTHX);
PERL_CALLCONV OP *Perl_pp_lc(pTHX);
PERL_CALLCONV OP *Perl_pp_le(pTHX);
PERL_CALLCONV OP *Perl_pp_leave(pTHX);
PERL_CALLCONV OP *Perl_pp_leaveeval(pTHX);
PERL_CALLCONV OP *Perl_pp_leavegiven(pTHX);
PERL_CALLCONV OP *Perl_pp_leaveloop(pTHX);
PERL_CALLCONV OP *Perl_pp_leavesub(pTHX);
PERL_CALLCONV OP *Perl_pp_leavesublv(pTHX);
PERL_CALLCONV OP *Perl_pp_leavetry(pTHX);
PERL_CALLCONV OP *Perl_pp_leavewhen(pTHX);
PERL_CALLCONV OP *Perl_pp_leavewrite(pTHX);
PERL_CALLCONV OP *Perl_pp_left_shift(pTHX);
PERL_CALLCONV OP *Perl_pp_length(pTHX);
PERL_CALLCONV OP *Perl_pp_link(pTHX);
PERL_CALLCONV OP *Perl_pp_list(pTHX);
PERL_CALLCONV OP *Perl_pp_listen(pTHX);
PERL_CALLCONV OP *Perl_pp_lock(pTHX);
PERL_CALLCONV OP *Perl_pp_lslice(pTHX);
PERL_CALLCONV OP *Perl_pp_lt(pTHX);
PERL_CALLCONV OP *Perl_pp_lvavref(pTHX);
PERL_CALLCONV OP *Perl_pp_lvref(pTHX);
PERL_CALLCONV OP *Perl_pp_lvrefslice(pTHX);
PERL_CALLCONV OP *Perl_pp_mapwhile(pTHX);
PERL_CALLCONV OP *Perl_pp_match(pTHX);
PERL_CALLCONV OP *Perl_pp_method(pTHX);
PERL_CALLCONV OP *Perl_pp_method_named(pTHX);
PERL_CALLCONV OP *Perl_pp_method_redir(pTHX);
PERL_CALLCONV OP *Perl_pp_method_redir_super(pTHX);
PERL_CALLCONV OP *Perl_pp_method_super(pTHX);
PERL_CALLCONV OP *Perl_pp_mkdir(pTHX);
PERL_CALLCONV OP *Perl_pp_modulo(pTHX);
PERL_CALLCONV OP *Perl_pp_multiconcat(pTHX);
PERL_CALLCONV OP *Perl_pp_multideref(pTHX);
PERL_CALLCONV OP *Perl_pp_multiply(pTHX);
PERL_CALLCONV OP *Perl_pp_nbit_and(pTHX);
PERL_CALLCONV OP *Perl_pp_nbit_or(pTHX);
PERL_CALLCONV OP *Perl_pp_ncmp(pTHX);
PERL_CALLCONV OP *Perl_pp_ncomplement(pTHX);
PERL_CALLCONV OP *Perl_pp_ne(pTHX);
PERL_CALLCONV OP *Perl_pp_negate(pTHX);
PERL_CALLCONV OP *Perl_pp_next(pTHX);
PERL_CALLCONV OP *Perl_pp_nextstate(pTHX);
PERL_CALLCONV OP *Perl_pp_not(pTHX);
PERL_CALLCONV OP *Perl_pp_null(pTHX);
PERL_CALLCONV OP *Perl_pp_oct(pTHX);
PERL_CALLCONV OP *Perl_pp_once(pTHX);
PERL_CALLCONV OP *Perl_pp_open(pTHX);
PERL_CALLCONV OP *Perl_pp_open_dir(pTHX);
PERL_CALLCONV OP *Perl_pp_or(pTHX);
PERL_CALLCONV OP *Perl_pp_ord(pTHX);
PERL_CALLCONV OP *Perl_pp_pack(pTHX);
PERL_CALLCONV OP *Perl_pp_padav(pTHX);
PERL_CALLCONV OP *Perl_pp_padcv(pTHX);
PERL_CALLCONV OP *Perl_pp_padhv(pTHX);
PERL_CALLCONV OP *Perl_pp_padrange(pTHX);
PERL_CALLCONV OP *Perl_pp_padsv(pTHX);
PERL_CALLCONV OP *Perl_pp_pipe_op(pTHX);
PERL_CALLCONV OP *Perl_pp_pos(pTHX);
PERL_CALLCONV OP *Perl_pp_postdec(pTHX);
PERL_CALLCONV OP *Perl_pp_postinc(pTHX);
PERL_CALLCONV OP *Perl_pp_pow(pTHX);
PERL_CALLCONV OP *Perl_pp_predec(pTHX);
PERL_CALLCONV OP *Perl_pp_preinc(pTHX);
PERL_CALLCONV OP *Perl_pp_print(pTHX);
PERL_CALLCONV OP *Perl_pp_prototype(pTHX);
PERL_CALLCONV OP *Perl_pp_prtf(pTHX);
PERL_CALLCONV OP *Perl_pp_push(pTHX);
PERL_CALLCONV OP *Perl_pp_pushmark(pTHX);
PERL_CALLCONV OP *Perl_pp_qr(pTHX);
PERL_CALLCONV OP *Perl_pp_quotemeta(pTHX);
PERL_CALLCONV OP *Perl_pp_rand(pTHX);
PERL_CALLCONV OP *Perl_pp_range(pTHX);
PERL_CALLCONV OP *Perl_pp_rcatline(pTHX);
PERL_CALLCONV OP *Perl_pp_readdir(pTHX);
PERL_CALLCONV OP *Perl_pp_readline(pTHX);
PERL_CALLCONV OP *Perl_pp_readlink(pTHX);
PERL_CALLCONV OP *Perl_pp_redo(pTHX);
PERL_CALLCONV OP *Perl_pp_ref(pTHX);
PERL_CALLCONV OP *Perl_pp_refassign(pTHX);
PERL_CALLCONV OP *Perl_pp_refgen(pTHX);
PERL_CALLCONV OP *Perl_pp_regcomp(pTHX);
PERL_CALLCONV OP *Perl_pp_regcreset(pTHX);
PERL_CALLCONV OP *Perl_pp_rename(pTHX);
PERL_CALLCONV OP *Perl_pp_repeat(pTHX);
PERL_CALLCONV OP *Perl_pp_require(pTHX);
PERL_CALLCONV OP *Perl_pp_reset(pTHX);
PERL_CALLCONV OP *Perl_pp_return(pTHX);
PERL_CALLCONV OP *Perl_pp_reverse(pTHX);
PERL_CALLCONV OP *Perl_pp_rewinddir(pTHX);
PERL_CALLCONV OP *Perl_pp_right_shift(pTHX);
PERL_CALLCONV OP *Perl_pp_rmdir(pTHX);
PERL_CALLCONV OP *Perl_pp_runcv(pTHX);
PERL_CALLCONV OP *Perl_pp_rv2av(pTHX);
PERL_CALLCONV OP *Perl_pp_rv2cv(pTHX);
PERL_CALLCONV OP *Perl_pp_rv2gv(pTHX);
PERL_CALLCONV OP *Perl_pp_rv2sv(pTHX);
PERL_CALLCONV OP *Perl_pp_sassign(pTHX);
PERL_CALLCONV OP *Perl_pp_sbit_and(pTHX);
PERL_CALLCONV OP *Perl_pp_sbit_or(pTHX);
PERL_CALLCONV OP *Perl_pp_schop(pTHX);
PERL_CALLCONV OP *Perl_pp_scmp(pTHX);
PERL_CALLCONV OP *Perl_pp_scomplement(pTHX);
PERL_CALLCONV OP *Perl_pp_seekdir(pTHX);
PERL_CALLCONV OP *Perl_pp_select(pTHX);
PERL_CALLCONV OP *Perl_pp_semctl(pTHX);
PERL_CALLCONV OP *Perl_pp_semget(pTHX);
PERL_CALLCONV OP *Perl_pp_seq(pTHX);
PERL_CALLCONV OP *Perl_pp_setpgrp(pTHX);
PERL_CALLCONV OP *Perl_pp_setpriority(pTHX);
PERL_CALLCONV OP *Perl_pp_shift(pTHX);
PERL_CALLCONV OP *Perl_pp_shmwrite(pTHX);
PERL_CALLCONV OP *Perl_pp_shostent(pTHX);
PERL_CALLCONV OP *Perl_pp_shutdown(pTHX);
PERL_CALLCONV OP *Perl_pp_sin(pTHX);
PERL_CALLCONV OP *Perl_pp_sle(pTHX);
PERL_CALLCONV OP *Perl_pp_sleep(pTHX);
PERL_CALLCONV OP *Perl_pp_smartmatch(pTHX);
PERL_CALLCONV OP *Perl_pp_sne(pTHX);
PERL_CALLCONV OP *Perl_pp_socket(pTHX);
PERL_CALLCONV OP *Perl_pp_sockpair(pTHX);
PERL_CALLCONV OP *Perl_pp_sort(pTHX);
PERL_CALLCONV OP *Perl_pp_splice(pTHX);
PERL_CALLCONV OP *Perl_pp_split(pTHX);
PERL_CALLCONV OP *Perl_pp_sprintf(pTHX);
PERL_CALLCONV OP *Perl_pp_srand(pTHX);
PERL_CALLCONV OP *Perl_pp_srefgen(pTHX);
PERL_CALLCONV OP *Perl_pp_sselect(pTHX);
PERL_CALLCONV OP *Perl_pp_ssockopt(pTHX);
PERL_CALLCONV OP *Perl_pp_stat(pTHX);
PERL_CALLCONV OP *Perl_pp_stringify(pTHX);
PERL_CALLCONV OP *Perl_pp_stub(pTHX);
PERL_CALLCONV OP *Perl_pp_study(pTHX);
PERL_CALLCONV OP *Perl_pp_subst(pTHX);
PERL_CALLCONV OP *Perl_pp_substcont(pTHX);
PERL_CALLCONV OP *Perl_pp_substr(pTHX);
PERL_CALLCONV OP *Perl_pp_subtract(pTHX);
PERL_CALLCONV OP *Perl_pp_syscall(pTHX);
PERL_CALLCONV OP *Perl_pp_sysopen(pTHX);
PERL_CALLCONV OP *Perl_pp_sysread(pTHX);
PERL_CALLCONV OP *Perl_pp_sysseek(pTHX);
PERL_CALLCONV OP *Perl_pp_system(pTHX);
PERL_CALLCONV OP *Perl_pp_syswrite(pTHX);
PERL_CALLCONV OP *Perl_pp_tell(pTHX);
PERL_CALLCONV OP *Perl_pp_telldir(pTHX);
PERL_CALLCONV OP *Perl_pp_tie(pTHX);
PERL_CALLCONV OP *Perl_pp_tied(pTHX);
PERL_CALLCONV OP *Perl_pp_time(pTHX);
PERL_CALLCONV OP *Perl_pp_tms(pTHX);
PERL_CALLCONV OP *Perl_pp_trans(pTHX);
PERL_CALLCONV OP *Perl_pp_truncate(pTHX);
PERL_CALLCONV OP *Perl_pp_uc(pTHX);
PERL_CALLCONV OP *Perl_pp_ucfirst(pTHX);
PERL_CALLCONV OP *Perl_pp_umask(pTHX);
PERL_CALLCONV OP *Perl_pp_undef(pTHX);
PERL_CALLCONV OP *Perl_pp_unpack(pTHX);
PERL_CALLCONV OP *Perl_pp_unshift(pTHX);
PERL_CALLCONV OP *Perl_pp_unstack(pTHX);
PERL_CALLCONV OP *Perl_pp_untie(pTHX);
PERL_CALLCONV OP *Perl_pp_vec(pTHX);
PERL_CALLCONV OP *Perl_pp_wait(pTHX);
PERL_CALLCONV OP *Perl_pp_waitpid(pTHX);
PERL_CALLCONV OP *Perl_pp_wantarray(pTHX);
PERL_CALLCONV OP *Perl_pp_warn(pTHX);
PERL_CALLCONV OP *Perl_pp_xor(pTHX);
PERL_CALLCONV OP *Perl_unimplemented_op(pTHX);

/* alternative functions */
#if defined(__GLIBC__) && IVSIZE == 8  && ( __GLIBC__ < 2 || (__GLIBC__ == 2 && __GLIBC_MINOR__ < 8))
PERL_CALLCONV OP *Perl_pp_i_modulo_glibc_bugfix(pTHX);
#endif

/* ex: set ro: */
