+{
   locale_version => 1.27,
   entry => <<'ENTRY', # for DUCET v10.0.0
0111      ; [.1CF5.0021.0002] # LATIN SMALL LETTER D WITH STROKE
0064 0335 ; [.1CF5.0021.0002] # LATIN SMALL LETTER D WITH STROKE
0110      ; [.1CF5.0021.0008] # LATIN CAPITAL LETTER D WITH STROKE
0044 0335 ; [.1CF5.0021.0008] # LATIN CAPITAL LETTER D WITH STROKE
00F0      ; [.1CF5.0022.0002] # LATIN SMALL LETTER ETH
1DD9      ; [.1CF5.0022.0002] # COMBINING LATIN SMALL LETTER ETH
00D0      ; [.1CF5.0022.0008] # LATIN CAPITAL LETTER ETH
00FE      ; [.1EFB.0020.0003][.1D7E.0020.0003] # LATIN SMALL LETTER THORN
00DE      ; [.1EFB.0020.0009][.1D7E.0020.0009] # LATIN CAPITAL LETTER THORN
00FC      ; [.1F71.0021.0002] # LATIN SMALL LETTER U WITH DIAERESIS
0075 0308 ; [.1F71.0021.0002] # LATIN SMALL LETTER U WITH DIAERESIS
00DC      ; [.1F71.0021.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
0055 0308 ; [.1F71.0021.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
01DC      ; [.1F71.0021.0002][.0000.0025.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE
01DB      ; [.1F71.0021.0008][.0000.0025.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE
01D8      ; [.1F71.0021.0002][.0000.0024.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE
01D7      ; [.1F71.0021.0008][.0000.0024.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE
01D6      ; [.1F71.0021.0002][.0000.0032.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND MACRON
01D5      ; [.1F71.0021.0008][.0000.0032.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON
01DA      ; [.1F71.0021.0002][.0000.0028.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND CARON
01D9      ; [.1F71.0021.0008][.0000.0028.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON
0171      ; [.1F71.0022.0002] # LATIN SMALL LETTER U WITH DOUBLE ACUTE
0075 030B ; [.1F71.0022.0002] # LATIN SMALL LETTER U WITH DOUBLE ACUTE
0170      ; [.1F71.0022.0008] # LATIN CAPITAL LETTER U WITH DOUBLE ACUTE
0055 030B ; [.1F71.0022.0008] # LATIN CAPITAL LETTER U WITH DOUBLE ACUTE
00E6      ; [.1FFC.0020.0002] # LATIN SMALL LETTER AE
1DD4      ; [.1FFC.0020.0002] # COMBINING LATIN SMALL LETTER AE
00C6      ; [.1FFC.0020.0008] # LATIN CAPITAL LETTER AE
1D2D      ; [.1FFC.0020.0014] # MODIFIER LETTER CAPITAL AE
01FD      ; [.1FFC.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER AE WITH ACUTE
01FC      ; [.1FFC.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER AE WITH ACUTE
01E3      ; [.1FFC.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER AE WITH MACRON
01E2      ; [.1FFC.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER AE WITH MACRON
00E4      ; [.1FFC.0021.0002] # LATIN SMALL LETTER A WITH DIAERESIS
0061 0308 ; [.1FFC.0021.0002] # LATIN SMALL LETTER A WITH DIAERESIS
00C4      ; [.1FFC.0021.0008] # LATIN CAPITAL LETTER A WITH DIAERESIS
0041 0308 ; [.1FFC.0021.0008] # LATIN CAPITAL LETTER A WITH DIAERESIS
01DF      ; [.1FFC.0021.0002][.0000.0032.0002] # LATIN SMALL LETTER A WITH DIAERESIS AND MACRON
01DE      ; [.1FFC.0021.0008][.0000.0032.0002] # LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON
0119      ; [.1FFC.0022.0002] # LATIN SMALL LETTER E WITH OGONEK
0065 0328 ; [.1FFC.0022.0002] # LATIN SMALL LETTER E WITH OGONEK
0118      ; [.1FFC.0022.0008] # LATIN CAPITAL LETTER E WITH OGONEK
0045 0328 ; [.1FFC.0022.0008] # LATIN CAPITAL LETTER E WITH OGONEK
00F8      ; [.1FFD.0020.0002] # LATIN SMALL LETTER O WITH STROKE
006F 0338 ; [.1FFD.0020.0002] # LATIN SMALL LETTER O WITH STROKE
00D8      ; [.1FFD.0020.0008] # LATIN CAPITAL LETTER O WITH STROKE
004F 0338 ; [.1FFD.0020.0008] # LATIN CAPITAL LETTER O WITH STROKE
01FF      ; [.1FFD.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER O WITH STROKE AND ACUTE
01FE      ; [.1FFD.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER O WITH STROKE AND ACUTE
00F6      ; [.1FFD.0021.0002] # LATIN SMALL LETTER O WITH DIAERESIS
006F 0308 ; [.1FFD.0021.0002] # LATIN SMALL LETTER O WITH DIAERESIS
00D6      ; [.1FFD.0021.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
004F 0308 ; [.1FFD.0021.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
022B      ; [.1FFD.0021.0002][.0000.0032.0002] # LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
022A      ; [.1FFD.0021.0008][.0000.0032.0002] # LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON
0151      ; [.1FFD.0022.0002] # LATIN SMALL LETTER O WITH DOUBLE ACUTE
006F 030B ; [.1FFD.0022.0002] # LATIN SMALL LETTER O WITH DOUBLE ACUTE
0150      ; [.1FFD.0022.0008] # LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
004F 030B ; [.1FFD.0022.0008] # LATIN CAPITAL LETTER O WITH DOUBLE ACUTE
0153      ; [.1FFD.0023.0002] # LATIN SMALL LIGATURE OE
0152      ; [.1FFD.0023.0008] # LATIN CAPITAL LIGATURE OE
00E5      ; [.1FFE.0020.0002] # LATIN SMALL LETTER A WITH RING ABOVE
0061 030A ; [.1FFE.0020.0002] # LATIN SMALL LETTER A WITH RING ABOVE
00C5      ; [.1FFE.0020.0008] # LATIN CAPITAL LETTER A WITH RING ABOVE
0041 030A ; [.1FFE.0020.0008] # LATIN CAPITAL LETTER A WITH RING ABOVE
212B      ; [.1FFE.0020.0008] # ANGSTROM SIGN
01FB      ; [.1FFE.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE
01FA      ; [.1FFE.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE
0061 0061 ; [.1FFE.0021.0002] # <LATIN SMALL LETTER A, LATIN SMALL LETTER A>
0041 0061 ; [.1FFE.0021.0007] # <LATIN CAPITAL LETTER A, LATIN SMALL LETTER A>
0041 0041 ; [.1FFE.0021.0008] # <LATIN CAPITAL LETTER A, LATIN CAPITAL LETTER A>
ENTRY
};
